#ifndef GEOMETRY_HH
#define GEOMETRY_HH

#include <cmath>

struct vec2 {
  float x, y;
  vec2(float x, float y) : x(x), y(y) {
  }
  float length() const { return sqrt(x*x + y*y); }
  vec2 operator*(float s) const { return vec2(s * x, s * y); }
  vec2 operator+(vec2 v) const { return vec2(x + v.x, y + v.y); }
  vec2 operator-(vec2 v) const { return vec2(x - v.x, y - v.y); }
  float distance(vec2 p) const { return (*this - p).length(); }
  vec2 normalized() const { return *this * (1 / length()); }
};

struct vec3 {
  float x, y, z;
  vec3(float x, float y, float z) : x(x), y(y), z(z) {
  }
  float length() const { return sqrt(x*x + y*y + z*z); }
  vec3 operator*(float s) const { return vec3(s * x, s * y, s * z); }
  vec3 operator+(vec3 v) const { return vec3(x + v.x, y + v.y, z + v.z); }
  vec3 operator-(vec3 v) const { return vec3(x - v.x, y - v.y, z - v.z); }
  vec3 normalized() const { return *this * (1 / length()); }
  vec3 cross_prod(const vec3 &v) const {
    return vec3(y * v.z - z * v.y,
                z * v.x - x * v.z,
                x * v.y - y * v.x);
  }
};



#endif
