#include "graphics.hh"
#ifdef OSX
#include <glut.h>
#else
#include <GL/glut.h>
#endif

namespace {
  int width, height;
  World *world;
}

void set_rock() {
  GLfloat mat_diffuse[] = { 1.0, 0.95, 0.7, 1.0 };
  GLfloat mat_shininess[] = { 50.0 };

  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_diffuse);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
}

void set_water() {
  GLfloat mat_diffuse[] = { 0.3, 0.3, 1.0, 1.0 };
  GLfloat mat_ambient[] = { 0, 0, 0 };
  GLfloat mat_shininess[] = { 100.0 };

  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
}

void draw() {
  const float scale = 0.05 * 100 / world->xsize;
  const float zscale = 0.07;
  const int xoff = -world->xsize / 2;
  const int yoff = -world->ysize / 2;
  const float min_water_depth = 0.00;

  for(int y = 0; y < world->ysize - 1; y++) {
    glBegin(GL_TRIANGLE_STRIP);
    for(int x = 0; x < world->xsize; x++) {
      vec3 n = world->normal(x, y + 1);
      float h1 = world->get_height(x, y + 1);
      float h2 = world->get_height(x, y);
      bool w1 = world->water.height(x, y + 1) > min_water_depth;
      bool w2 = world->water.height(x, y) > min_water_depth;
//       if(w1)
//         set_water();
//       else
//         set_rock();
      glNormal3f(n.x, n.y, n.z);
      glVertex3f((x + xoff) * scale, (y + 1 + yoff) * scale, h1 * zscale);
      n = world->normal(x, y);
//       if(w2)
//         set_water();
//       else
//         set_rock();
      glNormal3f(n.x, n.y, n.z);
      glVertex3f((x + xoff) * scale, (y + yoff) * scale, h2 * zscale);
    }
    glEnd();
  }
}

void display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // place camera
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(55, 1.0 * width / height, 0.5, 30.0);
  gluLookAt(0, -2.5, 2.5, // camera pos
            0, -1, 0,   // look at
            0, 0, 1);    // up

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  draw();

  glutSwapBuffers();
}

void reshape(int w, int h) {
  width = w; height = h;
  glViewport(0, 0, w, h);
}

void animate(int) {
  world->update();
  glutPostRedisplay();
  glutTimerFunc(10, animate, 0);
}

void graphics::init(int *argc, char **argv, int win_width, int win_height) {
  glutInit(argc, argv);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  //glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  width = win_width; height = win_height;
  glutInitWindowSize(width, height);
  glutCreateWindow("Sierra");
//   glEnable(GL_BLEND);
//   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glClearColor(0.4, 0.4, 0.7, 0.0);
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutTimerFunc(10, animate, 0);

  GLfloat light_position[] = { -30.0, -30.0, 30.0, 0.0 };
  GLfloat light_ambient[] = { 0.2, 0.1, 0.0, 1.0 };
  GLfloat light_diffuse[] = { 0.8, 0.8, 0.7, 1.0 };
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);

  set_rock();

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
}

void graphics::main_loop(World *world) {
  ::world = world;
  glutMainLoop();
}
