#include "heightfield.hh"
#include <stdexcept>
#include "geometry.hh"

HeightField::HeightField(int xsize, int ysize)
  : xsize(xsize), ysize(ysize), _height(new float[xsize * ysize])
{
  for(int x = 0; x < xsize; x++)
    for(int y = 0; y < ysize; y++)
      height(x, y) = 0;
}

HeightField::~HeightField() {
  delete _height;
}

float &HeightField::height(int x, int y) {
#ifdef DEBUG
  if(!inside(x, y))
    throw std::runtime_error("get_height(): not inside");
#endif
  return _height[y * ysize + x];
}

float &HeightField::height(int x, int y) const {
#ifdef DEBUG
  if(!inside(x, y))
    throw std::runtime_error("get_height(): not inside");
#endif
  return _height[y * ysize + x];
}

bool HeightField::inside(int x, int y) const {
  return x >= 0 && x < xsize && y >= 0 && y < ysize;
}


namespace {
  int round(float x) { return int(x + 0.5); }
}

void HeightField::erode(float x, float y, float radius, float depth) {
  const int minx = std::max(round(x - radius), 0);
  const int miny = std::max(round(y - radius), 0);
  const int maxx = std::min(round(x + radius), xsize);
  const int maxy = std::min(round(y + radius), ysize);
  const vec2 c(x, y); 
  for(int xx = minx; xx <= maxx; xx++)
    for(int yy = miny; yy <= maxy; yy++)
      height(xx, yy) -= depth * (radius - c.distance(vec2(xx, yy))) / radius;
}
